using System;
using System.Linq;
using System.Collections;
using System.Collections.Generic;
using MeshVR;
using MacGruber;
using UnityEngine;
using SimpleJSON;
using MVR.FileManagementSecure;


namespace CheesyFX {
	public partial class CheesyFX_WorkOut : MVRScript {
        TextureManager textureManager;
        public List<Rigidbody> forceReceivers = new List<Rigidbody>();
        float movementIntensity;
        public override void Init(){
            FileManagerSecure.CreateDirectory(BASE_DIRECTORY);
            loadURL = new JSONStorableUrl("loadURL", "", UILoadJSON, "jpg", true);
            loadURL.hideExtension = true;
            loadURL.setCallbackFunction -= UILoadJSON;
            loadURL.allowFullComputerBrowse = false;
            loadURL.allowBrowseAboveSuggestedPath = true;
            loadURL.SetFilePath(BASE_DIRECTORY+"/");
            loadURL.setCallbackFunction += UILoadJSON;
            foreach(ForceReceiver fr in containingAtom.forceReceivers){
				Rigidbody rb = fr.GetComponent<Rigidbody>();
				forceReceivers.Add(rb);
			}
            textureManager = new TextureManager(containingAtom);
            textureManager.drivers.Add(new FloatDriver(GetMovementIntensity, textureManager.glossJSON));
            CreateUI();
        }

        public float GetMovementIntensity(){
			float intensity = 0f;
			foreach(Rigidbody rb in forceReceivers){
				intensity += rb.mass * (rb.GetAcceleration() + rb.GetAngularAcceleration()).magnitude;
			}
			movementIntensity = intensity / 1000f;
			return movementIntensity;
		}

        

        public void Update(){
            textureManager.drivers.ForEach(x => x.Update());
        }
    }

    public class TextureManager
	{
		public Atom atom;
		public JSONStorable skin;
		public JSONStorableFloat glossJSON;
		public List<FloatDriver> drivers = new List<FloatDriver>();

		public TextureManager(Atom atom){
            this.atom = atom;
			skin = GetStorable("skin");
			glossJSON = skin.GetFloatJSONParam("Gloss");
		}

        public JSONStorable GetStorable(string name){
			string storableId = atom.GetStorableIDs().FirstOrDefault(x => x.Contains(name));
			if(storableId != null){
				return atom.GetStorableByID(storableId);
			}
			else return null;
		}
	}

	public class FloatDriver
	{
		public JSONStorableBool enabled = new JSONStorableBool("enabled", true);
		Func<float> GetRate;
        public JSONStorableFloat currentRate = new JSONStorableFloat("activeness", 0f, 0f, 50f, false, false);
        public float growthThreshold;
		public JSONStorableFloat receiver;
		public JSONStorableFloat maxGloss = new JSONStorableFloat("maxGloss", 8f, 4.7f, 8f, true, true);
		public JSONStorableFloat minGloss = new JSONStorableFloat("minGloss", 4.7f, 0f, 7.5f, true, true);
		public JSONStorableFloat highTreshold = new JSONStorableFloat("highThreshold", 2f, 0f, 5f, true, true);
		public JSONStorableFloat idleThreshold = new JSONStorableFloat("idleThreshold", .4f, 0f, 8f, true, true);
		public JSONStorableFloat timeEffect = new JSONStorableFloat("timeEffect", .5f, 0f, 5f, true, true);
		public float timeActive = 0f;

		public FloatDriver(Func<float> GetRate, JSONStorableFloat receiver){
			this.GetRate = GetRate;
			this.receiver = receiver;
		}

        public void SetToDefault(){
            maxGloss.val = maxGloss.defaultVal;
            minGloss.val = minGloss.defaultVal;
            highTreshold.val = highTreshold.defaultVal;
            idleThreshold.val = idleThreshold.defaultVal;
            timeEffect.val = timeEffect.defaultVal;
        }

		
		public void Update(){
			if(enabled.val){
                currentRate.val = GetRate();
				if(currentRate.val > idleThreshold.val) timeActive += Time.deltaTime;
				else if(timeActive > 0f) timeActive -= 3f*Time.deltaTime;
				float highT;
				if(timeEffect.val == 0f) highT = highTreshold.val;
				else highT = idleThreshold.val + (highTreshold.val)/(timeActive*timeEffect.val/100f+(highTreshold.val)/(highTreshold.val-idleThreshold.val));
				growthThreshold = idleThreshold.val + highT*(receiver.val - minGloss.val) / (maxGloss.val - minGloss.val);
				float goal = currentRate.val > growthThreshold? maxGloss.val : minGloss.val;
				
				receiver.val = Mathf.LerpUnclamped(receiver.val, goal, .01f * Time.deltaTime * Math.Abs(currentRate.val- growthThreshold));
			}
		}

        public JSONClass GetJSON(){
            JSONClass jc = new JSONClass();
            if(maxGloss.val != maxGloss.defaultVal) jc[maxGloss.name].AsFloat = maxGloss.val;
            if(minGloss.val != minGloss.defaultVal) jc[minGloss.name].AsFloat = minGloss.val;
            if(highTreshold.val != highTreshold.defaultVal) jc[highTreshold.name].AsFloat = highTreshold.val;
            if(idleThreshold.val != idleThreshold.defaultVal) jc[idleThreshold.name].AsFloat = idleThreshold.val;
            if(timeEffect.val != timeEffect.defaultVal) jc[timeEffect.name].AsFloat = timeEffect.val;
            return jc;
        }

        public void LoadJSON(JSONClass jc){
            if(jc.HasKey(maxGloss.name)) maxGloss.val = jc[maxGloss.name].AsFloat;
            if(jc.HasKey(minGloss.name)) minGloss.val = jc[minGloss.name].AsFloat;
            if(jc.HasKey(highTreshold.name)) highTreshold.val = jc[highTreshold.name].AsFloat;
            if(jc.HasKey(idleThreshold.name)) idleThreshold.val = jc[idleThreshold.name].AsFloat;
            if(jc.HasKey(timeEffect.name)) timeEffect.val = jc[timeEffect.name].AsFloat;
        }
    }

    public static class RigidbodyExt{
		static Vector3 lastVelocity = new Vector3();
		static Vector3 lastAngularVelocity = new Vector3();
		public static Vector3 GetAcceleration(this Rigidbody rb){
			Vector3 acceleration = (rb.velocity - lastVelocity) / Time.deltaTime;
			lastVelocity = rb.velocity;
			return acceleration;
		}
		public static Vector3 GetAngularAcceleration(this Rigidbody rb){
			Vector3 angularAcceleration = (rb.angularVelocity - lastAngularVelocity) / Time.deltaTime;
			lastAngularVelocity = rb.angularVelocity;
			return angularAcceleration;
		}
	}
}